@@ -14,7 +14,10 @@ sub init {
       temperature_subsystem => undef,
       cpu_subsystem => undef,
       memory_subsystem => undef,
+      nic_subsystem => undef,
       disk_subsystem => undef,
+      asr_subsystem => undef,
+      event_subsystem => undef,
   };
   $self->{serial} = 'unknown';
   $self->{product} = 'unknown';
@@ -29,6 +32,7 @@ sub init {
     $self->analyze_fan_subsystem();
     $self->analyze_temperatures();
     $self->analyze_memory_subsystem();
+    $self->analyze_nic_subsystem();
     $self->analyze_disk_subsystem();
     $self->analyze_asr_subsystem();
     $self->analyze_event_subsystem();
@@ -38,6 +42,7 @@ sub init {
     $self->check_fan_subsystem();
     $self->check_temperatures();
     $self->check_memory_subsystem();
+    $self->check_nic_subsystem();
     $self->check_disk_subsystem();
     $self->check_asr_subsystem();
     $self->check_event_subsystem();
@@ -46,6 +51,10 @@ sub init {
 
 sub identify {
   my $self = shift;
+  foreach (qw(product serial romversion)) {
+    $self->{$_} =~ s/^\s+//;
+    $self->{$_} =~ s/\s+$//;
+  }
   return sprintf "System: '%s', S/N: '%s', ROM: '%s'", 
       $self->{product}, $self->{serial}, $self->{romversion};
 }
@@ -121,6 +130,17 @@ sub analyze_memory_subsystem {
   );
 }
 
+sub analyze_nic_subsystem {
+  my $self = shift;
+  return if $self->{method} ne "snmp";
+  $self->{components}->{nic_subsystem} = 
+      HP::Proliant::Component::NicSubsystem->new(
+    rawdata => $self->{rawdata},
+    method => $self->{method},
+    runtime => $self->{runtime},
+  );
+}
+
 sub analyze_disk_subsystem {
   my $self = shift;
   $self->{components}->{disk_subsystem} =
@@ -186,6 +206,15 @@ sub check_memory_subsystem {
       if $self->{runtime}->{options}->{verbose} >= 2;
 }
 
+sub check_nic_subsystem {
+  my $self = shift;
+  return if $self->{method} ne "snmp";
+  if ($self->{runtime}->{plugin}->{opts}->get('eval-nics')) {
+    $self->{components}->{nic_subsystem}->check();
+    $self->{components}->{nic_subsystem}->dump()
+        if $self->{runtime}->{options}->{verbose} >= 2;
+  }
+}
 sub check_disk_subsystem {
   my $self = shift;
   $self->{components}->{disk_subsystem}->check();
@@ -296,7 +325,7 @@ EOEO
         $self->check_hpasm_client($hpasmcli);
         if (! $self->{runtime}->{plugin}->check_messages()) {
           foreach my $component (qw(server fans temp dimm powersupply iml)) {
-            if (open HPASMCLI, "$hpasmcli -s \"show $component\"|") {
+            if (open HPASMCLI, "$hpasmcli -s \"show $component\" </dev/null |") {
               my @output = <HPASMCLI>;
               close HPASMCLI;
               $self->{rawdata} .= join("\n", map {
@@ -325,13 +354,6 @@ EOEO
               $self->trace(2, sprintf "calling %s\n", $hpacucli);
               $self->check_hpacu_client($hpacucli);
               if (! $self->{runtime}->{plugin}->check_messages()) {
-                if (open HPACUCLI, "$hpacucli ctrl all show config 2>&1|") {
-                  my @output = <HPACUCLI>;
-                  close HPACUCLI;
-                  $self->{rawdata} .= join("\n", map {
-                      'config '.$_;
-                  } @output);
-                }
                 if (open HPACUCLI, "$hpacucli ctrl all show status 2>&1|") {
                   my @output = <HPACUCLI>;
                   close HPACUCLI;
@@ -339,6 +361,32 @@ EOEO
                       'status '.$_;
                   } @output);
                 }
+                if (open HPACUCLI, "$hpacucli ctrl all show config 2>&1|") {
+                  my @output = <HPACUCLI>;
+                  close HPACUCLI;
+                  $self->{rawdata} .= join("\n", map {
+                      'config '.$_;
+                  } @output);
+                  if (grep /Syntax error at "config"/, @output) {
+                    # older version of hpacucli CLI 7.50.18.0
+                    foreach my $slot (0..10) {
+                      if (open HPACUCLI, "$hpacucli ctrl slot=$slot logicaldrive all show 2>&1|") {
+                        my @output = <HPACUCLI>;
+                        close HPACUCLI;
+                        $self->{rawdata} .= join("\n", map {
+                            'config '.$_;
+                        } @output);
+                      }
+                      if (open HPACUCLI, "$hpacucli ctrl slot=$slot physicaldrive all show 2>&1|") {
+                        my @output = <HPACUCLI>;
+                        close HPACUCLI;
+                        $self->{rawdata} .= join("\n", map {
+                            'config '.$_;
+                        } @output);
+                      }
+                    }
+                  }
+                }
               } elsif ($self->{runtime}->{options}->{hpacucli} == 2) {
                 # we probably don't have sudo-privileges, but we were compiled with
                 # --enable-hpacucli=maybe
@@ -453,7 +501,7 @@ sub check_hpacu_client {
     } elsif (grep /(asswor[dt]:)|(You must be root)/, @output) {
       $self->add_message(UNKNOWN,
           sprintf "insufficient rights to call %s", $hpacucli);
-    } elsif (! grep /CLI Syntax/, @output) {
+    } elsif (! grep /(CLI Syntax)|(ACU CLI)/, @output) {
       $self->add_message(UNKNOWN,
           sprintf "insufficient rights to call %s", $hpacucli);
     }
@@ -477,6 +525,9 @@ sub set_serial {
   $self->{serial} = $self->{serial};
   $self->{product} = lc $self->{product};
   $self->{romversion} = $self->{romversion};
+  foreach (qw(serial product romversion)) {
+    $self->{$_} =~ s/\s+$//g;
+  }
 }
 
 
@@ -489,6 +540,36 @@ our @ISA = qw(HP::Proliant);
 
 sub collect {
   my $self = shift;
+  my %oidtables = (
+      system =>            "1.3.6.1.2.1.1",
+      cpqSeProcessor =>    "1.3.6.1.4.1.232.1.2.2",
+      cpqHePWSComponent => "1.3.6.1.4.1.232.6.2.9",
+      cpqHeThermal =>      "1.3.6.1.4.1.232.6.2.6",
+      cpqHeMComponent =>   "1.3.6.1.4.1.232.6.2.14",
+      cpqDaComponent =>    "1.3.6.1.4.1.232.3.2",
+      cpqSiComponent =>    "1.3.6.1.4.1.232.2.2",
+      cpqSeRom =>          "1.3.6.1.4.1.232.1.2.6",
+      cpqSasComponent =>   "1.3.6.1.4.1.232.5",
+      cpqIdeComponent =>   "1.3.6.1.4.1.232.14",
+      cpqFcaComponent =>   "1.3.6.1.4.1.232.16.2",
+      cpqHeAsr =>          "1.3.6.1.4.1.232.6.2.5",
+      cpqNic =>            "1.3.6.1.4.1.232.18.2",
+      cpqHeEventLog =>     "1.3.6.1.4.1.232.6.2.11",
+
+      #    cpqHeComponent =>  "1.3.6.1.4.1.232.6.2",
+      #    cpqHeFComponent => "1.3.6.1.4.1.232.6.2.6.7",
+      #    cpqHeTComponent => "1.3.6.1.4.1.232.6.2.6.8",
+  );
+  my %oidvalues = (
+      cpqHeEventLogSupported => "1.3.6.1.4.1.232.6.2.11.1.0",
+      cpqHeEventLogCondition => "1.3.6.1.4.1.232.6.2.11.2.0",
+      cpqNicIfLogMapOverallCondition => "1.3.6.1.4.1.232.18.2.2.2.0",
+      cpqHeThermalTempStatus => "1.3.6.1.4.1.232.6.2.6.3.0",
+      cpqHeThermalSystemFanStatus => "1.3.6.1.4.1.232.6.2.6.4.0",
+      cpqHeThermalCpuFanStatus => "1.3.6.1.4.1.232.6.2.6.5.0",
+      cpqHeAsrStatus => "1.3.6.1.4.1.232.6.2.5.1.0",
+      cpqHeAsrCondition => "1.3.6.1.4.1.232.6.2.5.17.0",
+  );
   if ($self->{runtime}->{plugin}->opts->snmpwalk) {
     my $cpqSeMibCondition = '1.3.6.1.4.1.232.1.1.3.0'; # 2=ok
     my $cpqHeMibCondition = '1.3.6.1.4.1.232.6.1.3.0'; # hat nicht jeder
@@ -501,6 +582,27 @@ sub collect {
         $self->add_message(CRITICAL,
             'snmpwalk returns no health data (cpqhlth-mib)');
     }
+    $self->{fullrawdata} = {};
+    %{$self->{fullrawdata}} = %{$self->{rawdata}};
+    $self->{rawdata} = {};
+    if (! $self->{runtime}->{plugin}->check_messages()) {
+      # for a better simulation, only put those oids into 
+      # rawdata which would also be put by a real snmp agent.
+      foreach my $table (keys %oidtables) {
+        my $oid = $oidtables{$table};
+        $oid =~ s/\./\\./g;
+        my $tmpoids = {};
+        my $tic = time;
+        map { $tmpoids->{$_} = $self->{fullrawdata}->{$_} }
+            grep /^$oid/, %{$self->{fullrawdata}};
+        my $tac = time;
+        $self->trace(2, sprintf "%03d seconds for walk %s (%d oids)",
+            $tac - $tic, $table, scalar(keys %{$tmpoids}));
+        map { $self->{rawdata}->{$_} = $tmpoids->{$_} } keys %{$tmpoids};
+      }
+      my @oids = values %oidvalues;
+      map { $self->{rawdata}->{$_} = $self->{fullrawdata}->{$_} } @oids;
+    }
   } else {
     my $net_snmp_version = Net::SNMP->VERSION(); # 5.002000 or 6.000000
     #$params{'-translate'} = [
@@ -512,6 +614,7 @@ sub collect {
       $self->{plugin}->add_message(CRITICAL, 'cannot create session object');
       $self->trace(1, Data::Dumper::Dumper($self->{runtime}->{snmpparams}));
     } else {
+      $session->translate(['-timeticks' => 0]);
       # revMajor is often used for discovery of hp devices
       my $cpqHeMibRev = '1.3.6.1.4.1.232.6.1';
       my $cpqHeMibRevMajor = '1.3.6.1.4.1.232.6.1.1.0';
@@ -541,164 +644,34 @@ sub collect {
       # snmp peer is alive
       $self->trace(2, sprintf "Protocol is %s", 
           $self->{runtime}->{snmpparams}->{'-version'});
-      my $cpqStdEquipment = "1.3.6.1.4.1.232";
-      my $cpqSeProcessor =  "1.3.6.1.4.1.232.1.2.2";
-      my $cpqSeRom =        "1.3.6.1.4.1.232.1.2.6";
-      my $cpqHeComponent =  "1.3.6.1.4.1.232.6.2";
-      my $cpqHePWSComponent = "1.3.6.1.4.1.232.6.2.9";
-      my $cpqHeThermal = "1.3.6.1.4.1.232.6.2.6";
-      #my $cpqHeFComponent = "1.3.6.1.4.1.232.6.2.6.7";
-      #my $cpqHeTComponent = "1.3.6.1.4.1.232.6.2.6.8";
-      my $cpqHeMComponent = "1.3.6.1.4.1.232.6.2.14";
-      my $cpqDaComponent =  "1.3.6.1.4.1.232.3.2";
-      my $cpqSasComponent =  "1.3.6.1.4.1.232.5";
-      my $cpqIdeComponent =  "1.3.6.1.4.1.232.14";
-      my $cpqFcaComponent =  "1.3.6.1.4.1.232.16.2";
-      my $cpqSiComponent =  "1.3.6.1.4.1.232.2.2";
-      my $cpqHeAsr = "1.3.6.1.4.1.232.6.2.5";
       $session->translate;
       my $response = {}; #break the walk up in smaller pieces
-      my $tic = time; my $tac = $tic;
-      my $response1 = $session->get_table(
-          -baseoid => $cpqSeProcessor);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqSeProcessor (%d oids)",
-          $tac - $tic, scalar(keys %{$response1}));
-      # Walk for PowerSupply
-      $tic = time;
-      my $response2p = $session->get_table(
-          -maxrepetitions => 1,
-          -baseoid => $cpqHePWSComponent);
-      if (scalar (keys %{$response2p}) == 0) {
-        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-        $response2p = $session->get_table(
-            -baseoid => $cpqHePWSComponent);
+      foreach my $table (keys %oidtables) {
+        my $oid = $oidtables{$table};
+        my $tic = time;
+        my $tmpresponse = $session->get_table(
+            -baseoid => $oid);
+        if (scalar (keys %{$tmpresponse}) == 0) {
+          $self->trace(2, sprintf "maxrepetitions failed. fallback");
+          $tmpresponse = $session->get_table(
+              -maxrepetitions => 1,
+              -baseoid => $oid);
+        }
+        my $tac = time;
+        $self->trace(2, sprintf "%03d seconds for walk %s (%d oids)",
+            $tac - $tic, $table, scalar(keys %{$tmpresponse}));
+        map { $response->{$_} = $tmpresponse->{$_} } keys %{$tmpresponse};
       }
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqHePWSComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response2p}));
-
-      # Walk for Fans/Temp/Overall
-      $tic = time;
-      my $response2f = $session->get_table(
-          -maxrepetitions => 1,
-          -baseoid => $cpqHeThermal);
-      if (scalar (keys %{$response2f}) == 0) {
-        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-        $response2f = $session->get_table(
-            -baseoid => $cpqHeThermal);
-      }
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqHeThermal (%d oids)",
-          $tac - $tic, scalar(keys %{$response2f}));
-
-#      # Walk for Fans
-#      $tic = time;
-#      my $response2f = $session->get_table(
-#          -maxrepetitions => 1,
-#          -baseoid => $cpqHeFComponent);
-#      if (scalar (keys %{$response2f}) == 0) {
-#        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-#        $response2f = $session->get_table(
-#            -baseoid => $cpqHeFComponent);
-#      }
-#      $tac = time;
-#      $self->trace(2, sprintf "%03d seconds for walk cpqHeFComponent (%d oids)",
-#          $tac - $tic, scalar(keys %{$response2f}));
-#      # Walk for Temp
-#      $tic = time;
-#      my $response2t = $session->get_table(
-#          -maxrepetitions => 1,
-#          -baseoid => $cpqHeTComponent);
-#      if (scalar (keys %{$response2t}) == 0) {
-#        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-#        $response2t = $session->get_table(
-#            -baseoid => $cpqHeTComponent);
-#      }
-#      $tac = time;
-#      $self->trace(2, sprintf "%03d seconds for walk cpqHeTComponent (%d oids)",
-#          $tac - $tic, scalar(keys %{$response2t}));
-      # Walk for Mem
-      $tic = time;
-      my $response2m = $session->get_table(
-          -maxrepetitions => 1,
-          -baseoid => $cpqHeMComponent);
-      if (scalar (keys %{$response2m}) == 0) {
-        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-        $response2m = $session->get_table(
-            -baseoid => $cpqHeMComponent);
-      }
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqHeMComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response2m}));
-      #
-      $tic = time;
-      my $response3 = $session->get_table(
-          -baseoid => $cpqDaComponent);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqDaComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response3}));
-      $tic = time;
-      my $response4 = $session->get_table(
-          -baseoid => $cpqSiComponent);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqSiComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response4}));
-      $tic = time;
-      my $response5 = $session->get_table(
-          -baseoid => $cpqSeRom);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqSeRom (%d oids)",
-          $tac - $tic, scalar(keys %{$response5}));
-      $tic = time;
-      my $response6 = $session->get_table(
-          -baseoid => $cpqSasComponent);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqSasComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response6}));
-      $tic = time;
-      my $response7 = $session->get_table(
-          -baseoid => $cpqIdeComponent);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqIdeComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response7}));
-      $tic = time;
-      my $response8 = $session->get_table(
-          -baseoid => $cpqFcaComponent);
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk cpqFcaComponent (%d oids)",
-          $tac - $tic, scalar(keys %{$response8}));
-      $tic = time;
-
-      # Walk for ASR
-      $tic = time;
-      my $response9 = $session->get_table(
-          -maxrepetitions => 1,
-          -baseoid => $cpqHeAsr);
-      if (scalar (keys %{$response9}) == 0) {
-        $self->trace(2, sprintf "maxrepetitions failed. fallback");
-        $response9 = $session->get_table(
-            -baseoid => $cpqHeAsr);
-      }
-      $tac = time;
-      $self->trace(2, sprintf "%03d seconds for walk $cpqHeAsr (%d oids)",
-          $tac - $tic, scalar(keys %{$response9}));
+      my @oids = values %oidvalues;
+      my $tic = time;
+      my $tmpresponse = $session->get_request(
+          -varbindlist => \@oids,
+      );
+      my $tac = time;
+      $self->trace(2, sprintf "%03d seconds for get various (%d oids)",
+          $tac - $tic, scalar(keys %{$tmpresponse}));
+      map { $response->{$_} = $tmpresponse->{$_} } keys %{$tmpresponse};
       $session->close();
-
-      map { $response->{$_} = $response1->{$_} } keys %{$response1};
-      map { $response->{$_} = $response2p->{$_} } keys %{$response2p};
-      map { $response->{$_} = $response2f->{$_} } keys %{$response2f};
-#      map { $response->{$_} = $response2t->{$_} } keys %{$response2t};
-      map { $response->{$_} = $response2m->{$_} } keys %{$response2m};
-      map { $response->{$_} = $response3->{$_} } keys %{$response3};
-      map { $response->{$_} = $response4->{$_} } keys %{$response4};
-      map { $response->{$_} = $response5->{$_} } keys %{$response5};
-      map { $response->{$_} = $response6->{$_} } keys %{$response6};
-      map { $response->{$_} = $response7->{$_} } keys %{$response7};
-      map { $response->{$_} = $response8->{$_} } keys %{$response8};
-      map { $response->{$_} = $response9->{$_} } keys %{$response9};
-      map { $response->{$_} =~ s/^\s+//; $response->{$_} =~ s/\s+$//; }
-          keys %$response;
       $self->{rawdata} = $response;
     }
   }
